@@ -1052,7 +1052,7 @@ v5dstruct *v5dNewStruct( void )
 {
    v5dstruct *v;
 
-   v = (v5dstruct *) malloc( sizeof(v5dstruct) );
+   v = (v5dstruct *) FerMem_Malloc( sizeof(v5dstruct), __FILE__, __LINE__ );
    if (v) {
       v5dInitStruct(v);
    }
@@ -1067,8 +1067,8 @@ v5dstruct *v5dNewStruct( void )
 void v5dFreeStruct( v5dstruct* v )
 {
    /*assert( v5dVerifyStruct( v ) );*/
-   free( v );
-   v = 0;
+   FerMem_Free( v, __FILE__, __LINE__ );
+   v = NULL;
 }
 
 
@@ -1418,7 +1418,6 @@ static int read_comp_header( int f, v5dstruct *v )
       int i, j, it, iv, nl;
       int gridsize;
       float hgttop, hgtinc;
-      /*char *compgrid;*/
 
       if (id==0x80808080) {
          /* 20 vars, 300 times */
@@ -1495,8 +1494,6 @@ static int read_comp_header( int f, v5dstruct *v )
          v->MaxVal[i] = -999999.9;
       }
 
-      /*compgrid = (char *) malloc( gridsize );*/
-
       for (it=0; it<v->NumTimes; it++) {
          for (iv=0; iv<v->NumVars; iv++) {
             float ga, gb;
@@ -1518,8 +1515,6 @@ static int read_comp_header( int f, v5dstruct *v )
          }
       }
 
-      /*free( compgrid );*/
-
       /* done */
    }
    else if (id==0x80808082 || id==0x80808083) {
@@ -2227,7 +2222,7 @@ int v5dReadGrid( v5dstruct *v, int time, int var, float data[] )
    else if (v->CompressMode==4) {
       bytes = v->Nr * v->Nc * v->Nl[var] * sizeof(float);
    }
-   compdata = (void *) malloc( bytes );
+   compdata = FerMem_Malloc( bytes, __FILE__, __LINE__ );
    if (!compdata) {
       printf("Error in v5dReadGrid: out of memory (needed %d bytes)\n", bytes);
       return 0;
@@ -2243,7 +2238,7 @@ int v5dReadGrid( v5dstruct *v, int time, int var, float data[] )
                       compdata, ga, gb, data );
 
    /* free compdata */
-   free( compdata );
+   FerMem_Free( compdata, __FILE__, __LINE__ );
    return 1;
 }
 
@@ -2635,7 +2630,7 @@ int v5dWriteGrid( v5dstruct *v, int time, int var, const float data[] )
    else if (v->CompressMode==4) {
       bytes = v->Nr * v->Nc * v->Nl[var] * sizeof(float);
    }
-   compdata = (void *) malloc( bytes );
+   compdata = FerMem_Malloc( bytes, __FILE__, __LINE__ );
    if (!compdata) {
       printf("Error in v5dWriteGrid: out of memory (needed %d bytes)\n",
              bytes );
@@ -2656,7 +2651,7 @@ int v5dWriteGrid( v5dstruct *v, int time, int var, const float data[] )
    n = v5dWriteCompressedGrid( v, time, var, ga, gb, compdata );
 
    /* free compdata */
-   free( compdata );
+   FerMem_Free( compdata, __FILE__, __LINE__  );
 
    return n;
 }
