@@ -1,7 +1,8 @@
 /* Make sure Python.h is included first */
 #include <Python.h>
 #include <stdlib.h>
-#include "EF_Util.h"
+#include "ferret.h"
+#include "FerMem.h"
 
 /* 
  *  Make a copy of a string up to a given length
@@ -12,9 +13,9 @@ void FORTRAN(ef_put_string)(char* text, int* inlen, char** out_ptr)
    int i;
 
    if ( *out_ptr != NULL )
-      free(*out_ptr);
+      FerMem_Free(*out_ptr, __FILE__, __LINE__);
 
-   *out_ptr = (char *) malloc(sizeof(char) * (*inlen+1));
+   *out_ptr = (char *) FerMem_Malloc(sizeof(char) * (*inlen+1), __FILE__, __LINE__);
    if ( *out_ptr == NULL ) {
       abort();
    }
