@@ -2,7 +2,9 @@
 #include <Python.h>
 #include <stdlib.h>
 #include <string.h>
+#include "ferret.h"
 #include "EF_Util.h"
+#include "FerMem.h"
 
 /* 
  *  Make a copy of a null-terminated string
@@ -11,9 +13,9 @@
 void FORTRAN(ef_put_string_ptr)(char **in_ptr, char **out_ptr)
 {
   if ( *out_ptr != NULL )
-     free(*out_ptr);
+     FerMem_Free(*out_ptr, __FILE__, __LINE__);
 
-  *out_ptr = (char *) malloc(sizeof(char) * (strlen(*in_ptr)+1));
+  *out_ptr = (char *) FerMem_Malloc(sizeof(char) * (strlen(*in_ptr)+1), __FILE__, __LINE__);
   if ( *out_ptr == NULL ) {
      abort();
   }
