@@ -3,7 +3,7 @@
 
 #include <Python.h>
 
-#ifdef VERBOSEDEBUG
+#ifdef GRDELDEBUG
 #include <stdio.h>
 extern FILE *debuglogfile;
 #endif
@@ -91,6 +91,7 @@ grdelType grdelWindowCreate(const char *engine, int enginelen,
                             grdelBool rasteronly);
 const BindObj *grdelWindowVerify(grdelType window);
 grdelBool grdelWindowDelete(grdelType window);
+void grdelWindowFree(grdelType window);
 grdelBool grdelWindowClear(grdelType window, grdelType fillcolor);
 grdelBool grdelWindowRedraw(grdelType window, grdelType fillcolor);
 grdelBool grdelWindowSetImageName(grdelType window, const char *imagename,
@@ -209,8 +210,8 @@ grdelType grdelBrushVerify(grdelType brush, grdelType window);
 grdelBool grdelBrushReplaceColor(grdelType brush, grdelType color);
 grdelBool grdelBrushDelete(grdelType brush);
 
-grdelType grdelSymbol(grdelType window, const char *symbolname,
-               int symbolnamelen);
+grdelType grdelSymbol(grdelType window, const char *symbolname, int symbolnamelen,
+                      const float ptsx[], const float ptsy[], int numpts);
 grdelType grdelSymbolVerify(grdelType symbol, grdelType window);
 grdelBool grdelSymbolDelete(grdelType symbol);
 
@@ -236,7 +237,8 @@ void fgdbrush_(void **brush, void **window, void **color,
 void fgdbrushreplacecolor_(int *success, void **brush, void **color);
 void fgdbrushdel_(int *success, void **brush);
 
-void fgdsymbol_(void **symbol, void **window, char *symbolname, int *namelen);
+void fgdsymbol_(void **symbol, void **window, char *symbolname, int *namelen,
+                float ptsx[], float ptsy[], int *numpts);
 void fgdsymboldel_(int *success, void **symbol);
 
 /*
