@@ -14,6 +14,7 @@ import sys
 
 from multiprocessing import Pipe
 
+WINDOW_CLOSED_MESSAGE = 'Window was closed'
 
 class PipedViewer(object):
     '''
@@ -24,8 +25,7 @@ class PipedViewer(object):
         '''
         Create and start a PipedViewer of one of the supported viewer 
         types.  The viewer will probably not be displayed until the 
-        { "action":"show" } command is submitted to the viewer cmndpipe 
-        using submitCommand.
+        {"action":"show"} command is submitted to the viewer.
 
         Currently supported viewer types are:
             "PipedViewerPQ": PipedViewerPQ using PyQt5 or PyQt4
@@ -60,11 +60,18 @@ class PipedViewer(object):
 
     def checkForResponse(self, timeout = 0.0):
         '''
-        Check for a response from the viewer.  The argument timeout
-        (a number) is the maximum time in seconds to block (default:
-        0.0; returns immediately).  If timeout is None, it will block
-        until something is read.  Returns the response from the viewer,
-        or None if there was no response in the allotted time.
+        Check for a response from the viewer.  
+
+        Arguments: 
+            timeout: (number) maximum time in seconds to wait for a 
+                     response to appear.  Zero (default) does not wait;
+                     None waits indefinitely.
+        Returns: 
+            the response from the viewer, or None if there was no 
+            response in the allotted time.
+        Raises: 
+            IOError: if the response pipe is closed while waiting 
+                     for a response to appear.
         '''
         if self.__rspdrecvpipe.poll(timeout):
             myresponse = self.__rspdrecvpipe.recv()
@@ -72,18 +79,35 @@ class PipedViewer(object):
             myresponse = None
         return myresponse
 
-    def waitForViewerExit(self):
+    def exitViewer(self):
         '''
-        Wait for all the submitted commands to be consumed and the
-        viewer  to return.  The command { "action":"exit" } should
-        have been the last command submitted to the command pipe
-        before calling this method.
+        Submit the command {"action":"exit"}, read any responses,
+        and waits for the viewer to exit.
+
+        Returns: 
+            any responses from the viewer, or 
+            an empty string if there was no response.
         '''
+        self.__cmndsendpipe.send({'action':'exit'})
         self.__cmndsendpipe.close()
+        closingremarks = ''
+        try:
+            # Read everything from the response pipe until it is closed
+            while self.__rspdrecvpipe.poll(0.1):
+                if closingremarks:
+                    closingremarks += '\n'
+                closingremarks += self.__rspdrecvpipe.recv()
+        except Exception:
+            pass
         self.__rspdrecvpipe.close()
         self.__vprocess.join()
+        return closingremarks
 
     def getViewerExitCode(self):
+        '''
+        Returns: 
+            the viewer process exit code.
+        '''
         return self.__vprocess.exitcode
 
 
@@ -97,7 +121,7 @@ def _testviewers():
     drawcmnds.append( { "action":"setTitle", "title":"Tester" } )
     drawcmnds.append( { "action":"show" } )
     drawcmnds.append( { "action":"clear", "color":"black"} )
-    drawcmnds.append( { "action":"dpi"} )
+    drawcmnds.append( { "action":"screenInfo"} )
     drawcmnds.append( { "action":"antialias", "antialias":True } )
     drawcmnds.append( { "action":"resize",
                         "width":500,
@@ -223,12 +247,12 @@ def _testviewers():
     drawcmnds.append( { "action":"endView" } )
     drawcmnds.append( { "action":"show" } )
     testannotations = ( "The 1<sup>st</sup> CO<sub>2</sub> annotations line",
-                        "Another line with <i>lengthy</i> details that should " + \
-                        "wrap to a 2<sup>nd</sup> annotation line",
+                        "Another line with <i>lengthy</i> details that go on and on " + \
+                        "and on and should wrap to a 2<sup>nd</sup> annotation line",
                         "<b>Final</b> annotation line" )
 
     # Test each known viewer.
-    for viewername in ( "PipedViewerPQ", "PipedImagerPQ" ):
+    for viewername in ( "PipedViewerPQ", ):
         print("Testing Viewer %s" % viewername)
         # create the viewer
         pviewer = PipedViewer(viewername)
@@ -243,15 +267,16 @@ def _testviewers():
                               "vectsize":{"width":7.0, "height":7.0},
                               "rastsize":{"width":750, "height":750},
                               "annotations":testannotations } )
-        mydrawcmnds.append( { "action":"exit" } )
-        
+
         # submit the commands, pausing after each "show" command
         for cmd in mydrawcmnds:
             print("Command: %s" % str(cmd))
             pviewer.submitCommand(cmd)
-            response = pviewer.checkForResponse()
+            response = pviewer.checkForResponse(0.1)
             while response:
                 print("Response: %s" % str(response))
+                if response == WINDOW_CLOSED_MESSAGE:
+                    return
                 response = pviewer.checkForResponse()
             if cmd["action"] == "show":
                 if sys.version_info[0] > 2:
@@ -259,10 +284,12 @@ def _testviewers():
                 else:
                     raw_input("Press Enter to continue")
         # end of the commands - shut down and check return value
-        pviewer.waitForViewerExit()
+        response = pviewer.exitViewer()
+        if response:
+            print("Closing remarks: %s" % str(response))
         result = pviewer.getViewerExitCode()
         if result != 0:
-            sys.exit(result)
+            print("Exit code %i from %s" % (result, viewername))
         else:
             print("Done with %s" % viewername)
 
