@@ -1,5 +1,5 @@
 '''
-PipedViewerPQ is a graphics viewer application written in PyQt4
+PipedViewerPQ is a graphics viewer application written in PyQt
 that receives its drawing and other commands primarily from another
 application through a pipe.  A limited number of commands are
 provided by the viewer itself to allow saving and some manipulation
@@ -27,16 +27,11 @@ try:
 except AttributeError:
     pass
 
-if sys.version_info[0] > 2:
-    # First try to import PyQt5, then try PyQt4 if that fails
-    try:
-        import PyQt5
-        QT_VERSION = 5
-    except ImportError:
-        import PyQt4
-        QT_VERSION = 4
-else:
-    # PyQt5 requires Python3.x, so only try PyQt4
+# First try to import PyQt5, then try PyQt4 if that fails
+try:
+    import PyQt5
+    QT_VERSION = 5
+except ImportError:
     import PyQt4
     QT_VERSION = 4
 
@@ -56,12 +51,13 @@ else:
     from PyQt4.QtGui  import QAction, QApplication, QBrush, QColor, QDialog, \
                              QFileDialog, QFontMetricsF, QImage, QLabel, \
                              QMainWindow, QMessageBox, QPainter, QPalette, \
-			     QPen, QPicture, QPixmap, QPolygonF, QPrinter, \
-			     QPushButton, QScrollArea, QTextDocument
+                             QPen, QPicture, QPixmap, QPolygonF, QPrinter, \
+                             QPushButton, QScrollArea, QTextDocument
     from PyQt4.QtSvg  import QSvgGenerator
 
 from multiprocessing import Pipe, Process
 
+from pipedviewer import WINDOW_CLOSED_MESSAGE
 from pipedviewer.cmndhelperpq import CmndHelperPQ
 from pipedviewer.scaledialogpq import ScaleDialogPQ
 
@@ -149,8 +145,6 @@ class PipedViewerPQ(QMainWindow):
         self.__lastfilename = "ferret.png"
         self.__lastformat = "png"
         self.__addedannomargin = 12
-        # Control whether the window will be destroyed or hidden
-        self.__shuttingdown = False
         # command helper object
         self.__helper = CmndHelperPQ(self)
         # Create the menubar
@@ -226,42 +220,30 @@ class PipedViewerPQ(QMainWindow):
 
     def closeEvent(self, event):
         '''
-        Override so will just minimize if not shutting down
+        Clean up and send the WINDOW_CLOSED_MESSAGE on the response pipe 
+        before closing the window.
         '''
-        # If shutting down or minimized (escape hatch), 
-        # go ahead and close/exit
-        if self.__shuttingdown or self.isMinimized():
-            self.__timer.stop()
-            event.accept()
-            return
-        # Otherwise just minimize the window
-        event.ignore()
-        self.showMinimized()
+        self.__timer.stop()
+        self.__cmndpipe.close()
+        self.__rspdpipe.send(WINDOW_CLOSED_MESSAGE)
+        self.__rspdpipe.close()
+        event.accept()
 
     def exitViewer(self):
         '''
         Close and exit the viewer.
         '''
-        self.__shuttingdown = True
         self.close()
 
     def aboutMsg(self):
         QMessageBox.about(self, self.tr("About PipedViewerPQ"),
             self.tr("\n" \
-            "PipedViewerPQ is a graphics viewer application that " \
-            "receives its drawing and other commands primarily from " \
-            "another application through a pipe.  A limited number " \
-            "of commands are provided by the viewer itself to allow " \
-            "saving and some manipulation of the displayed image.  " \
-            "The controlling application, however, may be unaware " \
-            "of these modifications made to the image. " \
-            "\n\n" \
-            "'Closing' this window (the window frame 'X' button) " \
-            "actually only minimizes this viewer as PyFerret expects " \
-            "it to exist until PyFerret closes it.  Selecting the " \
-            "close menu option from the *minimized* window (using " \
-            "a right mouse click) will actually close (exit) the " \
-            "viewer if PyFerret exited and left it up.  " \
+            "PipedViewerPQ is a graphics viewer application that receives its " \
+            "drawing and other commands primarily from another application " \
+            "through a pipe.  A limited number of commands are provided by " \
+            "the viewer itself to allow saving and some manipulation of the " \
+            "displayed image.  The controlling application, however, may be " \
+            "unaware of these modifications made to the image. " \
             "\n\n" \
             "PipedViewerPQ was developed by the Thermal Modeling and Analysis " \
             "Project (TMAP) of the National Oceanographic and Atmospheric " \
@@ -630,8 +612,8 @@ class PipedViewerPQ(QMainWindow):
                         ( "xbm",
                           self.tr("XBM - X11 Bitmap (*.xbm)") ), ]
         if QT_VERSION == 5:
-            # tr returns Python strings in PyQt5/Python3
-            filters = ";;".join( [ t[1] for t in formattypes ] )
+            # tr returns Python unicode strings in PyQt5/Python3
+            filters = self.tr(";;").join( [ t[1] for t in formattypes ] )
             for (fmt, fmtQName) in formattypes:
                 if self.__lastformat == fmt:
                     dfltfilter = fmtQName
@@ -1567,9 +1549,7 @@ class PipedViewerPQProcess(Process):
         self.__app = QApplication(["PipedViewerPQ"])
         self.__viewer = PipedViewerPQ(self.__cmndpipe, self.__rspdpipe)
         myresult = self.__app.exec_()
-        self.__cmndpipe.close()
-        self.__rspdpipe.close()
-        SystemExit(myresult)
+        sys.exit(myresult)
 
 
 #
@@ -1604,7 +1584,7 @@ class _CommandSubmitterPQ(QDialog):
             print("Command: %s" % str(self.__cmndlist[self.__nextcmnd]))
             self.__cmndpipe.send(self.__cmndlist[self.__nextcmnd])
             self.__nextcmnd += 1
-            while self.__rspdpipe.poll():
+            while self.__rspdpipe.poll(0.1):
                 print("Response: %s" % str(self.__rspdpipe.recv()))
         except IndexError:
             self.__rspdpipe.close()
@@ -1759,8 +1739,8 @@ def _test_pipedviewerpq():
     drawcmnds.append( { "action":"update" } )
     drawcmnds.append( { "action":"show" } )
     annotations = ( "The 1<sup>st</sup> CO<sub>2</sub> annotations line",
-                    "Another line with <i>lengthy</i> details that should " + \
-                    "wrap to a 2<sup>nd</sup> annotation line",
+                    "Another line with <i>lengthy</i> details that go on and on " + \
+                    "and on and should wrap to a 2<sup>nd</sup> annotation line",
                     "<b>Final</b> annotation line" )
     drawcmnds.append( { "action":"save",
                         "filename":"test.pdf",
