@@ -20,7 +20,7 @@ from __future__ import print_function
 import sys
 
 from pyferret.graphbind.abstractpyferretbindings import AbstractPyFerretBindings
-from pipedviewer import PipedViewer
+from pipedviewer import PipedViewer, WINDOW_CLOSED_MESSAGE
 
 
 class PyFerretBindings(AbstractPyFerretBindings):
@@ -75,33 +75,12 @@ class PyFerretBindings(AbstractPyFerretBindings):
         self.checkForErrorResponse(self.__shortwait)
         return True
 
-    def submitCommand(self, cmnd):
-        '''
-        Submits the given command to PipedViewer.
-
-        Provided for use by functions defined in subclasses.
-        '''
-        # Check for an error from any previous command
-        self.__checkForErrorResponse()
-        # If all okay, submit the command
-        self.__window.submitCommand(cmnd)
-
-    def checkForResponse(self, timeout=0.0):
-        '''
-        Checks the response pipe for an object within the given timeout 
-        number of in seconds.  If timeout is None, this method will wait 
-        indefinitely for something to be given on the responds pipe.  If 
-        timeout is zero (the default) the method does not wait if there 
-        is no response waiting.  If nothing is obtained within the given 
-        timeout, None is returned.
-        '''
-        result = self.__window.checkForResponse(timeout)
-        return result
-
     def checkForErrorResponse(self, timeout=0.0):
         '''
-        Checks the response pipe for a message.  If anything is found,
-        a RuntimeError is raised with the string of the full response.
+        Checks the response pipe for a message that will be interpreted
+        as an error message.  If a response is found, a RuntimeError will
+        be raised with the string of the full response.
+
         If timeout is zero (the default) the method does not wait if there
         is no response waiting; otherwise the method waits the given number
         of seconds for a response to arrive.  An IllegalArgumentException
@@ -133,10 +112,7 @@ class PyFerretBindings(AbstractPyFerretBindings):
         Returns True.
         '''
         try:
-            self.__window.submitCommand( { "action":"exit" } )
-            # wait briefly for any error messages
-            self.checkForErrorResponse(self.__shortwait)
-            self.__window.waitForViewerExit()
+            closingremarks = self.__window.exitViewer()
         finally:
             self.__window = None
         return True
@@ -585,23 +561,34 @@ class PyFerretBindings(AbstractPyFerretBindings):
         '''
         del brush
 
-    def createSymbol(self, symbolname):
+    def createSymbol(self, symbolname, ptsx=None, ptsy=None):
         '''
         Returns a Symbol object.
 
         Arguments:
-            symbolname: name of the symbol.
-                Currently supported values are:
-                '.' (period): filled circle
-                'o' (lowercase oh): unfilled circle
-                '+': plus mark
-                'x' (lowercase ex): x mark
-                '*': asterisk
-                '^': triangle
-                "#": square
+            symbolname: name of the symbol, either a well-known
+                  symbol name (e.g., '.') or a custom name for a 
+                  symbol created from the given vertices (e.g., 'FER001')
+            ptsx: vertices X-coordinates describing the symbol 
+                  as a multiline drawing on a [0,100] square; 
+                  not used if a well-known symbol name is given
+            ptsy: vertices Y-coordinates describing the symbol 
+                  as a multiline drawing on a [0,100] square; 
+                  not used if a well-known symbol name is given
+        Currently supported well-known symbol names are:
+            '.' (period): filled circle
+            'o' (lowercase oh): unfilled circle
+            '+': plus mark
+            'x' (lowercase ex): x mark
+            '*': asterisk
+            '^': unfilled triangle
+            "#": unfilled square
 
-        Raises an error if unable to create the Symbol object.
+        Raises an error if ptsx and ptsy are needed but are not
+        sequence types of the same size or if unable to create 
+        the Symbol object for any other reason.
         '''
+        # TODO: support custom symbols
         return symbolname
 
     def deleteSymbol(self, symbol):
@@ -880,7 +867,7 @@ class PImagerPQPyFerretBindings(PyFerretBindings):
                  "width":width,
                  "height":height,
                  "stride":stride }
-        self.submitCommand(cmnd)
+        self.__window.submitCommand(cmnd)
         blocksize = 8192
         numblocks = (lenimgdata + blocksize - 1) // blocksize
         for k in range(numblocks):
@@ -893,7 +880,7 @@ class PImagerPQPyFerretBindings(PyFerretBindings):
                      "numblocks":numblocks,
                      "startindex":k*blocksize,
                      "blockdata":blkdata }
-            self.submitCommand(cmnd)
+            self.__window.submitCommand(cmnd)
         # wait briefly at the end for any error messages
         self.checkForErrorResponse(self.__shortwait)
 
@@ -934,12 +921,12 @@ def _test_pyferretbindings():
                 )
 
     # Initiate pyferret, but stay in python
-    pyferret.init(None, False)
+    pyferret.init(enterferret=False)
     for viewertype in ( "PipedViewerPQ", ):
         print("Testing bindings for %s" % viewertype)
         # Create a viewer window
-        title = viewertype + "Tester"
-        bindinst = pyferret.graphbind.createWindow(viewertype, title, True, False, False)
+        wintitle = viewertype + "Tester"
+        bindinst = pyferret.graphbind.createWindow(engine_name=viewertype, title=wintitle, visible=True, noalpha=False, rasteronly=False)
         # Resize the window to 500 x 500 pixels
         bindinst.resizeWindow(500, 500)
         # Turn on anti-aliasing
@@ -951,6 +938,11 @@ def _test_pyferretbindings():
                      for (r, g, b, a) in colorvals ]
         # Clear the window in black
         bindinst.clearWindow(mycolors[0])
+        bindinst.showWindow(True)
+        if sys.version_info[0] > 2:
+            input("Press Enter to continue")
+        else:
+            raw_input("Press Enter to continue")
         # Create a view in the bottom left corner
         bindinst.beginView(0.0, 1.0, 0.5, 0.0, True)
         # Draw a translucent green rectangle over most of the view
