@@ -2,10 +2,11 @@
 A Python module for running Ferret.
 For running the Ferret engine that is part of this module:
     init or start must first be called to initialize Ferret
-    resize can be used to resize Ferret's allocated memory block
+    resize can be used to reset the maximum amount of memory
+            that can be allocated for Ferret data
     run is used to submit individual Ferret commands or enter
             into Ferret's command prompting mode
-    stop can be used to shutdown Ferret and free the allocated
+    stop can be used to shutdown Ferret and free any allocated
             memory.
 
 The FERR_* values are the possible values of err_int in the return
@@ -86,58 +87,206 @@ from pyferret.fermethods import *
 # bindings for the PyQt-based graphics engines
 import pipedviewer.pyferretbindings
 
-from pyferret import filenamecompleter
 from pyferret import graphbind
 from pyferret import regrid
 
-def init(arglist=None, enterferret=True):
+
+def _addtoenvdirlist(envname, dirname):
     """
-    Interprets the traditional Ferret options given in arglist and
-    starts pyferret appropriately.  Defines all the standard Ferret
-    Python external functions.  If ${HOME}/.ferret exists, that
-    script is then executed.
-
-    If '-script' is given with a script filename, this method calls
-    the run method with the ferret go command, the script filename,
-    and any arguments, and then exits completely (exits python).
-
-    Otherwise, if enterferret is False, this method just returns the
-    success return value of the run method: (FERR_OK, '')
-
-    If enterferret is True (unless '-python' is given in arglist)
-    this routine calls the run method with no arguments in order to
-    enter into Ferret command-line processing.  The value returned
-    from call to the run method is then returned.
+    Adds the directory path given by dirname to the list of directory
+    paths given in the environment variable envname.  The directory 
+    path is added to the beginning of the environment value, but 
+    immediately after '.'  if that is the first directory in the list.
+    If the environment variable given in envname does not exist, it 
+    is initialized to '.' before adding the given directory path. 
+    No checking or modification of the directory path is performed.
     """
+    currval = os.getenv(envname, '.')
+    # Spaces in directory names are a problem for ferret 
+    # so do not worry about it here. 
+    dirlist = currval.split()
+    if dirlist[0] == '.':
+        dirlist.insert(1, dirname)
+    else:
+        dirlist.insert(0, dirname);
+    os.environ[envname] = ' '.join(dirlist)
 
-    ferret_help_message = \
+
+def addenv(FER_DIR=None, FER_DAT=None, FER_DATA=None, FER_DESCR=None, 
+           FER_GRIDS=None, FER_GO=None, FER_FONTS=None, FER_PALETTE=None):
     """
+    A convenience method that adds the specified directory paths to the 
+    current Ferret environment variables in the current Python session. 
+    Each value should be a single directory path.  More directories can 
+    be added to an environment variable, if appropriate, by calling this 
+    method again.  
+
+    Except for FER_DIR, FER_DAT, and FER_FONTS (see below), the directory 
+    specified will be added to the beginning of the list of directory paths 
+    given by the environment variable, but immediately after '.' (which 
+    represents the current working directory) if that is the first directory 
+    in the list.  If an enviroment variable is not defined, it will be 
+    initialized to '.' before adding the given directory path.
+
+    The value for FER_DIR, FER_DAT or FER_FONTS, if specified, replaces 
+    the current value (if any) for these environment variables. (These 
+    values are each suppose to be a single directory path).  
+
+    The value for FER_DIR (which actually is not used directly in PyFerret)
+    is also used to construct directory paths to be added, if the subdirectory 
+    exists, to the enviroment variables: 
+        FER_GO ("$FER_DIR/go"), 
+        FER_PALETTE ("$FER_DIR/ppl"), 
+        FER_FONTS ("$FER_DIR/ppl/fonts"), and 
+        LD_LIBRARY_PATH ("$FER_DIR/lib/pythonX.x/site-packages/pyferret", 
+                         where X and x are the major and minor version 
+                         numbers of this python). 
+    These directory paths will be added prior to any directories specified 
+    by other arguments.
+
+    The value for FER_DAT (which actually is not used directly in PyFerret) 
+    is also used to construct directory paths to be added, if the subdirectory 
+    exists, to the environment variables:
+        FER_DATA ("$FER_DAT/data"),
+        FER_DESCR ("$FER_DAT/descr"), and
+        FER_GRIDS ("$FER_DAT/grids")
+    These directory paths will be added prior to any directories specified 
+    by other arguments.
+
+    Directory paths can use '~/' and '~user/' constructs, as well as shell 
+    variables of the form '$var' and '${var}'.  Relative paths can also be 
+    specified and will be used as-is (they are not converted to absolute 
+    paths).
+
+    Use os.environ to reset a Ferret environment variable to some specific
+    value; for example, 
+        os.environ['FER_DATA'] = '. /path/to/my/datasets'
+
+    To replicate setting most of the environment variables assigned by the 
+    ferret_paths scripts, call this method with the FER_DIR and FER_DAT
+    arguments assigned the values used in generating the ferret_paths
+    scripts.  So a python session using pyferret without using the 
+    ferret_paths scripts will first require adding the 
+        pyferret/lib/pythonX.x/site-packages 
+    directory path, if not in a standard python location, to either the 
+    shell environment variable PYTHONPATH or the Python list of paths given 
+    in sys.path so the pyferret module can be found.  The rest of the
+    session will look something like:
+        import pyferret
+        pyferret.addenv(FER_DIR='/path/to/pyferret/dir', 
+                        FER_DAT='/path/to/ferret/datasets/dir')
+        pyferret.start()
+    """
+    if FER_DIR:
+        myferdir = os.path.expandvars(os.path.expanduser(FER_DIR))
+        if not os.path.isdir(myferdir):
+            raise ValueError('FER_DIR is given but is not a directory: %s' % myferdir)
+        os.environ['FER_DIR'] = myferdir
+        # Set FER_FONTS to ppl/fonts subdir
+        myferfonts = os.path.join(myferdir, 'ppl', 'fonts')
+        if os.path.isdir(myferfonts):
+            os.environ['FER_FONTS'] = myferfonts
+        # Add go subdir to FER_GO
+        dirtoadd = os.path.join(myferdir, 'go')
+        if os.path.isdir(dirtoadd):
+            _addtoenvdirlist('FER_GO', dirtoadd)
+        # Add ppl subdir to FER_PALETTE
+        dirtoadd = os.path.join(myferdir, 'ppl')
+        if os.path.isdir(dirtoadd):
+            _addtoenvdirlist('FER_PALETTE', dirtoadd)
+        # Add lib/pythonX.x/site-packages/pyferret subdir to LD_LIBRARY_PATH
+        # but separate with appropriate separator (':') and do not worry about '.'
+        dirtoadd = os.path.join(myferdir, 'lib', 'python%d.%d' % sys.version_info[:2], 
+                                'site-packages', 'pyferret')
+        currlist = os.getenv('LD_LIBRARY_PATH')
+        if not currlist:
+            os.environ['LD_LIBRARY_PATH'] = dirtoadd
+        else:
+            os.environ['LD_LIBRARY_PATH'] = dirtoadd + os.pathsep + currlist
+    if FER_DAT:
+        myferdat = os.path.expandvars(os.path.expanduser(FER_DAT))
+        if not os.path.isdir(myferdat):
+            raise ValueError('FER_DAT is given but is not a directory: %s' % myferdat)
+        os.environ['FER_DAT'] = myferdat
+        # Add data subdir to FER_DATA
+        dirtoadd = os.path.join(myferdat, 'data')
+        if os.path.isdir(dirtoadd):
+            _addtoenvdirlist('FER_DATA', dirtoadd)
+        # Add descr subdir to FER_DESCR
+        dirtoadd = os.path.join(myferdat, 'descr')
+        if os.path.isdir(dirtoadd):
+            _addtoenvdirlist('FER_DESCR', dirtoadd)
+        # Add grids subdir to FER_GRIDS
+        dirtoadd = os.path.join(myferdat, 'grids')
+        if os.path.isdir(dirtoadd):
+            _addtoenvdirlist('FER_GRIDS', dirtoadd)
+    if FER_FONTS:
+        myferfonts = os.path.expandvars(os.path.expanduser(FER_FONTS))
+        if not os.path.isdir(myferfonts):
+            raise ValueError('FER_FONTS is given but is not a directory: %s' % myferfonts)
+        os.environ['FER_FONTS'] = myferfonts
+    if FER_GO:
+        dirtoadd = os.path.expandvars(os.path.expanduser(FER_GO))
+        if not os.path.isdir(dirtoadd):
+            raise ValueError('FER_GO is given but is not a directory: %s' % dirtoadd)
+        _addtoenvdirlist('FER_GO', dirtoadd)
+    if FER_PALETTE:
+        dirtoadd = os.path.expandvars(os.path.expanduser(FER_PALETTE))
+        if not os.path.isdir(dirtoadd):
+            raise ValueError('FER_PALETTE is given but is not a directory: %s' % dirtoadd)
+        _addtoenvdirlist('FER_PALETTE', dirtoadd)
+    if FER_DATA:
+        dirtoadd = os.path.expandvars(os.path.expanduser(FER_DATA))
+        if not os.path.isdir(dirtoadd):
+            raise ValueError('FER_DATA is given but is not a directory: %s' % dirtoadd)
+        _addtoenvdirlist('FER_DATA', dirtoadd)
+    if FER_DESCR:
+        dirtoadd = os.path.expandvars(os.path.expanduser(FER_DESCR))
+        if not os.path.isdir(dirtoadd):
+            raise ValueError('FER_DESCR is given but is not a directory: %s' % dirtoadd)
+        _addtoenvdirlist('FER_DESCR', dirtoadd)
+    if FER_GRIDS:
+        dirtoadd = os.path.expandvars(os.path.expanduser(FER_GRIDS))
+        if not os.path.isdir(dirtoadd):
+            raise ValueError('FER_GRIDS is given but is not a directory: %s' % dirtoadd)
+        _addtoenvdirlist('FER_GRIDS', dirtoadd)
+
+
+def addpath(newdir):
+    """
+    A convenience method that replicates the Ferret Faddpath script by 
+    adding the given directory to the environment variables FER_GO, 
+    FER_DATA, FER_DESCR, and FER_GRIDS in the current Python session. 
+    """
+    addenv(FER_GO=newdir, FER_DATA=newdir, FER_DESCR=newdir, FER_GRIDS=newdir)
+
+
+pyferret_help_message = \
+"""
 
     Usage:  pyferret  [-memsize <N>]  [-nodisplay]  [-png]  [-nojnl]  [-noverify]
                       [-secure]  [-server]  [-python]  [-version]  [-help]
                       [-quiet]  [-linebuffer]  [-script <scriptname> [ <scriptarg> ... ]]
 
-       -memsize:     initialize the memory cache size to <N> (default 25)
-                     mega (10^6) floats (where 1 float = 8 bytes)
+       -memsize:     maximum amount of memory that can be  allocates for Ferret data 
+                     in mega (10^6) floats (where 1 float = 8 bytes); default is 25
 
-       -nodisplay    do not display to the console; a drawing can be saved
-                     using the FRAME command in any of the supported file
-                     formats.  The /QUALITY option of SET WINDOW will be
-                     ignored when this is specified.
+       -nodisplay    do not display to the console; a drawing can be saved using the 
+                     FRAME command in any of the supported file formats.  The /QUALITY 
+                     option of SET WINDOW will be ignored when this is specified.
 
-       -png          do not display to the console and only raster (PNG)
-                     images can be saved.  Implies -nodisplay and removes
-                     support for deletion of parts (segments) of a plot;
-                     however, plots will be generated faster.
+       -png          do not display to the console and only raster (PNG) images can be 
+                     saved.  Implies -nodisplay and removes support for deletion of 
+                     parts (segments) of a plot; however, plots will be generated faster.
 
-       -nojnl:       on startup do not open a journal file (can be turned
-                     on later with SET MODE JOURNAL)
+       -nojnl:       on startup do not open a journal file (can be turned on later with 
+                     SET MODE JOURNAL)
 
-       -noverify:    on startup turn off verify mode (can be turned on
-                     later with SET MODE VERIFY)
+       -noverify:    on startup turn off verify mode (can be turned on later with 
+                     SET MODE VERIFY)
 
-       -secure:      restrict Ferret's capabilities (e.g., SPAWN and
-                     EXIT /TOPYTHON are not permitted)
+       -secure:      restrict Ferret's capabilities (e.g., SPAWN and EXIT /TOPYTHON 
+                     are not permitted)
 
        -server:      run in server mode (don't stop on message commands)
 
@@ -150,37 +299,57 @@ def init(arglist=None, enterferret=True):
 
        -quiet        do not display the startup header
 
-       -linebuffer   print each line of output or error messages as soon as
-                     a full line is written.  Useful when redirecting both
-                     types of messages to a single file.
+       -linebuffer   print each line of output or error messages as soon as a full 
+                     line is written.  Useful when redirecting both types of messages 
+                     to a single file.
                      Note:
                        the environment variable GFORTRAN_UNBUFFERED_PRECONNECTED
                        needs to be set to 1 in order to unbuffer the Fortran
                        units for output and error messages
 
-       -script:      execute the script <scriptname> with any arguments
-                     specified and exit. (THIS MUST BE SPECIFIED LAST.)
-                     The -script option also implies the -nojnl, -noverify,
-                     -server, and -quiet options.
+       -script:      execute the script with any arguments specified and exit. 
+                     (THIS MUST BE SPECIFIED LAST.)  The -script option also implies 
+                     the -nojnl, -noverify, -server, and -quiet options.
 
        Deprecated command-line options:
 
            -gif:         this option is now an alias for -png
 
-	   -unmapped:    this option is now an alias for -nodisplay
+           -unmapped:    this option is now an alias for -nodisplay
 
-           -batch:       draw to <filename> (default "ferret.png") instead of
-                         displaying to the console.  The file format will be
-                         guessed from the filename extension.  When using this
-                         option, new windows should not be created and the
-                         FRAME command should not be used.
+           -batch:       name of file to draw to (default "ferret.png") instead of 
+                         displaying to the console.  The file format will be guessed 
+                         from the filename extension.  When using this option, new 
+                         windows should not be created and the FRAME command should 
+                         not be used.
 
-           -transparent: use a transparent background instead of opaque white
-                         when saving to the file given by -batch
+           -transparent: use a transparent background instead of opaque white when 
+                         saving to the file given by -batch
 
         Use of -batch and -transparent options is strongly discouraged.  Instead,
         use the -nodisplay option and the FRAME /FILE=<filename> [ /TRANSPARENT ]
         Ferret command.
+
+"""
+
+def init(arglist=None, enterferret=True):
+    """
+    Interprets the traditional Ferret options given in arglist and calls 
+    pyferret.start appropriately.  Detailed information about the strings 
+    that can be given in arglist can be obtained by printing the 
+    pyferret.pyferret_help_message string. 
+
+    If '-script' is given with a script filename, this method calls the 
+    pyferret.run method with the Ferret go command, the script filename, 
+    and any arguments, then exits completely (exits python).
+
+    Otherwise, if enterferret is False, this method just returns the success 
+    return value of the run method: (FERR_OK, '')
+
+    If enterferret is True (unless '-python' is given in arglist) this routine 
+    calls the run method with no arguments in order to enter into Ferret 
+    command-line processing.  The value returned from call to the run method 
+    is then returned when the Ferret command EXIT /TOPY is made.
     """
 
     my_metaname = None
@@ -278,16 +447,156 @@ def init(arglist=None, enterferret=True):
             print_help = True
         if print_help:
             # print the help message, then mark for exiting
-            print(ferret_help_message, file=sys.stderr)
+            print(pyferret_help_message, file=sys.stderr)
             just_exit = True
         if just_exit:
             # print the ferret header then exit completely
-            start(journal=False, verify=False, unmapped=True)
+            if not start(journal=False, verify=False, unmapped=True):
+                raise ValueError('pyferret has already been started')
             result = run("exit /program")
             # should not get here
             raise SystemExit
 
-    # Create the list of standard ferret PyEFs to create
+    # start ferret
+    if not start(memsize=my_memsize, journal=my_journal, verify=my_verify,
+                 restrict=my_restrict, server=my_server, unmapped=my_unmapped,
+                 pngonly=my_pngonly, quiet=my_quiet, linebuffer=my_linebuffer,
+                 metaname=my_metaname, transparent=my_transparent):
+        raise ValueError("pyferret has already been started")
+
+    # if a command-line script is given, run the script and exit completely
+    if script != None:
+        # put double quotes around every script argument
+        script_line = '"' + '" "'.join(script) + '"'
+        try:
+            result = run('go ' + script_line)
+        except:
+            print(" **Error: exception raised in running script %s" % script_line, file=sys.stderr)
+        # force shutdown
+        result = run('exit /program')
+        # should not get here
+        raise SystemExit
+
+    if my_restrict or my_enterferret:
+        # go into Ferret command-line processing until "exit /topy" (if not restricted) or "exit /program"
+        result = run()
+    else:
+        # if they don't want to enter ferret, just return the success value from run
+        result = (libpyferret.FERR_OK, '')
+    return result
+
+
+def start(memsize=25, journal=True, verify=False, restrict=False,
+          server=False, unmapped=False, pngonly=False, quiet=False, 
+          linebuffer=False, metaname=None, transparent=False):
+    """
+    Initializes Ferret from the arguments given.  
+    Executes any Ferret commands given in $HOME/.ferret
+
+    Arguments:
+        memsize: (number)     the maximum amount on memory, in mega (10^6) floats 
+                              (where 1 float = 8 bytes) that can be allocated for 
+                              data in Ferret.
+
+        journal: (boolean)    turn on Ferret's journal mode? (can be changed later 
+                              using SET or CANCEL MODE JOURNAL)
+
+        verify: (boolean)     turn on Ferret's verify mode? (can be changed later 
+                              using SET or CANCEL MODE VERIFY)
+
+        restrict: (boolean)   restrict Ferret's capabilities? (e.g., SPAWN and
+                              EXIT /TOPYTHON are not permitted)
+
+        server: (boolean)     run in server mode? (don't stop on message commands,
+                              don't enable command-line completion and history)
+
+        unmapped: (boolean)   if True, do not display to the console; the /QUALITY 
+                              option of SET WINDOW will be ignored.  A drawing can 
+                              be saved using the FRAME command in any of the supported 
+                              file formats. 
+
+        pngonly: (boolean)    if True, do not display to the console and only raster (PNG) 
+                              images can be saved; plots will be generated faster but 
+                              removes support for deletion of parts (segments) of a plot.
+
+        quiet: (boolean)      do not display the Ferret start-up header?
+
+        linebuffer: (boolean) if True, prints each line of an output or error message 
+                              as soon as a full line is written; useful when redirecting 
+                              these messages to a file.
+                              Note:
+                                the enviroment variable GFORTRAN_UNBUFFERED_PRECONNECTED
+                                needs to be set to 1 in order to unbuffer the Fortran
+                                units for output and error messages
+
+       Deprecated:
+           metaname: (string) name of file to draw to (default "ferret.png") instead of 
+                              displaying to the console.  The file format will be guessed 
+                              from the filename extension.  When using this option, new 
+                              windows should not be created and the FRAME command should 
+                              not be used.
+
+           transparent: (boolean) use a transparent background instead of opaque white 
+                                  when saving to the file given by metaname
+
+       Use of metaname and transparent is strongly discouraged.  Instead, set unmapped 
+       to True and use the FRAME /FILE=<filename> [ /TRANSPARENT ] Ferret command.
+
+    Returns:
+        True is successful
+        False if Ferret has already been started
+    Raises:
+        ValueError if memsize if not a positive number
+        IOError if unable to open the journal file
+    """
+    # check memsize
+    try:
+        flt_memsize = float(memsize)
+        if flt_memsize <= 0.0:
+            raise ValueError
+    except:
+        raise ValueError("memsize must be a positive number")
+    # check metaname
+    if metaname == None:
+        str_metaname = ""
+    elif not isinstance(metaname, str):
+        raise ValueError("metaname must either be None or a string")
+    elif metaname.isspace():
+        str_metaname = ""
+    else:
+        str_metaname = metaname
+
+    # Get the known viewer bindings
+    knownengines = graphbind.knownPyFerretEngines()
+    # Add PViewerPQPyFerretBindings, as "PipedViewerPQ" to the known bindings
+    if not ("PipedViewerPQ" in knownengines):
+        graphbind.addPyFerretBindings("PipedViewerPQ",
+                  pipedviewer.pyferretbindings.PViewerPQPyFerretBindings)
+    # Add PImagerPQPyFerretBindings, as "PipedImagerPQ" to the known bindings
+    if not ("PipedImagerPQ" in knownengines):
+        graphbind.addPyFerretBindings("PipedImagerPQ",
+                  pipedviewer.pyferretbindings.PImagerPQPyFerretBindings)
+
+    # the actual call to ferret's start - do not turn on journaling or verify at this time
+    if not libpyferret._start(flt_memsize, False, False,
+                              bool(restrict), bool(server), str_metaname,
+                              bool(transparent), bool(unmapped),
+                              bool(pngonly), bool(quiet), bool(linebuffer)):
+        return False
+
+    # register the libpyferret._quit function with atexit to ensure
+    # open viewer windows do not hang a Python shutdown
+    atexit.register(libpyferret._quit)
+
+    # Use tab completion for readline
+    if 'libedit' in readline.__doc__:
+        # Mac OS X actually uses libedit for system readline
+        readline.parse_and_bind('bind ^I rl_complete');
+    else:
+        # GNU readline
+        readline.parse_and_bind('tab: complete');
+
+    # Create the list of standard PyEFs to create
     std_pyefs = [ ]
     # stats_* functions that do not need scipy
     std_pyefs.append("stats.stats_histogram")
@@ -452,11 +761,7 @@ def init(arglist=None, enterferret=True):
                   "stats.stats_helper",
                   ))
     except ImportError:
-        # if not my_quiet:
-        #     print >>sys.stderr, "    WARNING: Unable to import scipy;\n" \
-        #                         "             most stats_* Ferret functions will not be added."
         pass
-
     # shapefile_* functions
     try:
         import shapefile
@@ -470,11 +775,7 @@ def init(arglist=None, enterferret=True):
                   "fershp.shapefile_writexyzval",
                   ))
     except ImportError:
-        # if not my_quiet:
-        #     print >>sys.stderr, "    WARNING: Unable to import shapefile;\n" \
-        #                         "             shapefile_* Ferret functions will not be added."
         pass
-
     # regrid functions
     try:
         import ESMP
@@ -483,24 +784,14 @@ def init(arglist=None, enterferret=True):
                   "regrid.curv3srect",
                   ))
     except ImportError:
-        # if not my_quiet:
-        #     print >>sys.stderr, "    WARNING: Unable to import ESMP;\n" \
-        #                         "             curv2rect* Ferret functions will not be added.\n" \
-        #                         "             Use curv_to_rect* functions instead"
         pass
 
-    # start ferret without journaling
-    start(memsize=my_memsize, journal=False, verify=my_verify,
-          restrict=my_restrict, server=my_server, metaname=my_metaname,
-          transparent=my_transparent, unmapped=my_unmapped,
-          pngonly=my_pngonly, quiet=my_quiet, linebuffer=my_linebuffer)
-
     # define all the Ferret standard Python external functions
     for fname in std_pyefs:
         result = run("define pyfunc pyferret.%s" % fname)
 
-    # run the ${HOME}/.ferret script if it exists and not restricted environment
-    if not my_restrict:
+    # run the ${HOME}/.ferret script if it exists and not in a restricted environment
+    if not restrict:
         home_val = os.getenv('HOME')
         if home_val:
             init_script = os.path.join(home_val, '.ferret')
@@ -513,151 +804,29 @@ def init(arglist=None, enterferret=True):
                     # should not get here
                     raise SystemExit
 
-    # if a command-line script is given, run the script and exit completely
-    if script != None:
-        # put double quotes around every script argument
-        script_line = '"' + '" "'.join(script) + '"'
-        try:
-            result = run('go %s; exit /program' % script_line)
-        except:
-            print(" **Error: exception raised in running script %s" % script_line, file=sys.stderr)
-        # If exception or if returned early, force shutdown
-        result = run('exit /program')
-        # should not get here
-        raise SystemExit
-
-    # if journaling desired, now turn on journaling
-    if my_journal:
+    # if journaling desired, turn it on now
+    if journal:
         result = run("set mode journal")
 
-    # if they don't want to enter ferret, return the success value from run
-    if not my_enterferret:
-        return (libpyferret.FERR_OK, '')
+    # if verify desired, turn it on now
+    if verify:
+        result = run("set mode verify")
 
-    # otherwise, go into Ferret command-line processing until "exit /topy" or "exit /program"
-    result = run()
-    return result
-
-
-def start(memsize=25, journal=True, verify=False, restrict=False,
-          server=False, metaname=None, transparent=False,
-          unmapped=False, pngonly=False, quiet=False, linebuffer=False):
-    """
-    Initializes Ferret.  This allocates the initial amount of memory
-    for Ferret (from Python-managed memory), opens the journal file,
-    if requested, and sets Ferret's verify mode.
-    If restrict is True, some Ferret commands will not be available
-    (to provide a secured session).  Once restrict is set, it cannot
-    be unset.
-    If metaname is not empty this value is used as the initial filename
-    for automatic output of graphics, and the graphics viewer will not
-    be displayed.
-    If server is True, Ferret will be run in server mode.
-    If unmapped is True, the graphics viewer will not be displayed.
-    If pngonly is True, only raster images can be saved and the
-    graphics viewer will not be displayed.
-    If quiet is True, the Ferret start-up header is not displayed.
-    If linebuffer is True, stdout and stderr are set user line
-    buffering.  This cannot be reset once set.
-    This routine does NOT run any user initialization scripts.
-
-    Arguments:
-        memsize:     the size, in mega (10^6) floats (where 1 float
-                     = 8 bytes) to allocate for Ferret's memory block
-        journal:     turn on Ferret's journal mode?
-        verify:      turn on Ferret's verify mode?
-        restrict:    restrict Ferret's capabilities?
-        server:      put Ferret in server mode?
-        metaname:    filename for Ferret graphics; can be None or empty
-        transparent: autosave (e.g., on exit) image files with a
-                     transparent background?
-        unmapped:    hide the graphics viewer?
-        pngonly:     only create raster (PNG) images?
-        quiet:       do not display the Ferret start-up header?
-        linebuffer:  print each line of output or error messages as soon as
-                     a full line is written?  Useful when redirecting these
-                     messages to a file.
-                     Note:
-                       the enviroment variable GFORTRAN_UNBUFFERED_PRECONNECTED
-                       needs to be set to 1 in order to unbuffer the Fortran
-                       units for output and error messages
-    Returns:
-        True is successful
-        False if Ferret has already been started
-    Raises:
-        ValueError if memsize if not a positive number
-        MemoryError if unable to allocate the needed memory
-        IOError if unable to open the journal file
-    """
-    # check memsize
-    try:
-        flt_memsize = float(memsize)
-        if flt_memsize <= 0.0:
-            raise ValueError
-    except:
-        raise ValueError("memsize must be a positive number")
-    # check metaname
-    if metaname == None:
-        str_metaname = ""
-    elif not isinstance(metaname, str):
-        raise ValueError("metaname must either be None or a string")
-    elif metaname.isspace():
-        str_metaname = ""
-    else:
-        str_metaname = metaname
-
-    # Get the known viewer bindings
-    knownengines = graphbind.knownPyFerretEngines()
-    # Add PViewerPQPyFerretBindings, as "PipedViewerPQ" to the known bindings
-    if not ("PipedViewerPQ" in knownengines):
-        graphbind.addPyFerretBindings("PipedViewerPQ",
-                  pipedviewer.pyferretbindings.PViewerPQPyFerretBindings)
-    # Add PImagerPQPyFerretBindings, as "PipedImagerPQ" to the known bindings
-    if not ("PipedImagerPQ" in knownengines):
-        graphbind.addPyFerretBindings("PipedImagerPQ",
-                  pipedviewer.pyferretbindings.PImagerPQPyFerretBindings)
-
-    # the actual call to ferret's start
-    success = libpyferret._start(flt_memsize, bool(journal), bool(verify),
-                                 bool(restrict), bool(server), str_metaname,
-                                 bool(transparent), bool(unmapped),
-                                 bool(pngonly), bool(quiet), bool(linebuffer))
-    if success:
-        # register the libpyferret._quit function with atexit to ensure
-        # open viewer windows do not hang a Python shutdown
-        atexit.register(libpyferret._quit)
-
-        # Use tab completion for readline (for Ferret) by default
-        readline.parse_and_bind('tab: complete');
-
-        # Execute the $PYTHONSTARTUP file, if it exists and -secure not given
-        if not restrict:
-            try:
-                startfilename = os.getenv('PYTHONSTARTUP', '')
-                if startfilename:
-                    if sys.version_info[0] > 2:
-                        exec(compile(open(startfilename).read(), startfilename, 'exec'));
-                    else:
-                        execfile(startfilename)
-            except Exception:
-                pass
-
-    return success
+    return True
 
 
 def resize(memsize):
     """
-    Resets the the amount of memory allocated for Ferret from Python-managed memory.
+    Resets the the maximum amount of memory that can be allocated for Ferret data.
 
     Arguments:
         memsize: the new size, in mega (10^) floats (where a "float" is 8 bytes),
-                 for Ferret's memory block
+                 for Ferret data.
     Returns:
-        True if successful - Ferret has the new amount of memory
-        False if unsuccessful - Ferret has the previous amount of memory
+        True if successful - Ferret has the new memory limit
+        False if unsuccessful - Ferret has the previous memory limit
     Raises:
         ValueError if memsize if not a positive number
-        MemoryError if Ferret has not been started or has been stopped
     """
     # check memsize
     try:
@@ -670,6 +839,7 @@ def resize(memsize):
     return libpyferret._run("SET MEMORY /SIZE=%d" % flt_memsize)
 
 
+FERRET_READLINE_DELIMS = ' \t\n\'"\\`@$><=;|&{('
 def run(command=None):
     """
     Runs a Ferret command just as if entering a command at the Ferret prompt.
@@ -677,7 +847,12 @@ def run(command=None):
     If the command is not given, is None, or is a blank string, Ferret will
     prompt you for commands until "EXIT /TOPYTHON" is given.  In this case,
     the return tuple will be for the last error, if any, that occurred in
-    the sequence of commands given to Ferret.
+    the sequence of commands given to Ferret.  Also, when at the Ferret 
+    prompt, the readline library's default filename completion will be used 
+    but with word delimiters given in pyferret.FERRET_READLINE_DELIMS. 
+    At startup the word delimiters are the readline default delimiters: 
+        ' \\t\\n\\'"\\\\`@$><=;|&{('
+    This string can be modified if desired.
 
     Arguments:
         command: the Ferret command to be executed.
@@ -689,7 +864,6 @@ def run(command=None):
         Warning messages normally start with "*** NOTE:"
     Raises:
         ValueError if command is neither None nor a String
-        MemoryError if Ferret has not been started or has been stopped
     """
     # check command
     if command == None:
@@ -703,15 +877,18 @@ def run(command=None):
     # if going into Ferret-command mode,
     # use the filename completer for readline name completion
     if str_command == "":
+        old_delims = readline.get_completer_delims()
         old_completer = readline.get_completer()
-        filename_completer_obj = filenamecompleter.FilenameCompleter()
-        readline.set_completer(filename_completer_obj.complete)
+        # set to the readline default completer (filename completion)
+        readline.set_completer(None)
+        # set the word delimitors appropriate for Ferret
+        readline.set_completer_delims(FERRET_READLINE_DELIMS)
     # the actual Ferret function call
     retval = libpyferret._run(str_command)
     # return to the original readline completer
     if str_command == "":
+        readline.set_completer_delims(old_delims)
         readline.set_completer(old_completer)
-        del filename_completer_obj
     return retval
 
 
