@@ -8,7 +8,8 @@ import numbers
 import pyferret
 
 
-def setwindow(num=1, plotasp=None, axisasp=None, color=None, thick=None, logo=None):
+def setwindow(num=1, plotasp=None, axisasp=None, color=None, pal=None, 
+              thick=None, logo=None, outline=None):
     """
     Assigns the plot window to use for subsequent plotting commands.
     Also provides assignment of common window plots.  
@@ -26,12 +27,19 @@ def setwindow(num=1, plotasp=None, axisasp=None, color=None, thick=None, logo=No
             of int values in [0,100] giving RGB or RGBA values.
             If not given, the current value is unchanged.
             The default background color on start-up is opaque white.
+        pal (string): default color palette to use in plots.
+            If not given, thr current value is unchanged.
         thick (float): line thickness scaling factor for the plot.
             If not given, the current scaling factor is unchanged.
             The default line thickness scaling factor on start-up is 1.0
         logo (boolean): include the Ferret logo in the plot?
             If not given, the current value is unchanged.
             The default on start-up is to include the logo.
+        outline (float): if positive, thickness of polygon outlines;
+            used to fix the 'thin white line' issue in plots.
+            If not given, the current value is unchanged.
+            The default on start-up is zero (no outlines drawn).
+            
     Raises a ValueError if a problem occurs.
     """
     # create and execute the SET WINDOW command
@@ -50,6 +58,10 @@ def setwindow(num=1, plotasp=None, axisasp=None, color=None, thick=None, logo=No
         if (not isinstance(thick, numbers.Real)) or (thick <= 0):
             raise ValueError('thick, if given, must be a positive number')
         cmdstr += '/THICK=' + str(thick)
+    if outline is not None:
+        if (not isinstance(outline, numbers.Real)) or (outline < 0):
+            raise ValueErrror('outline, if given, must be a non-negative number')
+        cmdstr += '/OUTLINE=' + str(outline)
     if color is not None:
         if isinstance(color, str):
             cmdstr += '/COLOR=' + color
@@ -65,6 +77,11 @@ def setwindow(num=1, plotasp=None, axisasp=None, color=None, thick=None, logo=No
     (errval, errmsg) = pyferret.run(cmdstr)
     if errval != pyferret.FERR_OK:
         raise ValueError('Problems executing Ferret command %s: %s' % (cmdstr, errmsg))
+    if pal is not None:
+        cmdstr = 'PALETTE ' + str(pal)
+        (errval, errmsg) = pyferret.run(cmdstr)
+        if errval != pyferret.FERR_OK:
+            raise ValueError('Problems executing Ferret command %s: %s' % (cmdstr, errmsg))
     # create and execute the mode logo command if logo is given
     if logo is not None:
         if logo:
@@ -129,7 +146,7 @@ def showdata(brief=True, qual=''):
         raise ValueError('Ferret command "%s" failed: %s' % (cmdstr, errmsg))
 
 
-def contourplot(fvar, region=None, over=False, qual=''):
+def contourplot(fvar, region=None, over=False, pal=None, qual=''):
     """
     Create a contour plot of the specified Ferret variable using the Ferret CONTOUR command.
     Using the fill method to generated a color-filled contour plot.
@@ -138,6 +155,7 @@ def contourplot(fvar, region=None, over=False, qual=''):
         region (FerRegion): space-time region to plot; 
                 if None, the full extents of the data will be used
         over (bool): overlay on an existing plot?
+        pal (string): color palette to use
         qual (string): qualifiers to add to the Ferret SHADE command
     """
     if not isinstance(qual, str):
@@ -155,6 +173,8 @@ def contourplot(fvar, region=None, over=False, qual=''):
         if not isinstance(region, pyferret.FerRegion):
             raise ValueError('region, if given, must be a FerRegion')
         cmdstr += region._ferretqualifierstr();
+    if pal is not None:
+        cmdstr += '/PALETTE=' + str(pal)
     if qual:
         cmdstr += qual
     cmdstr += ' '
@@ -164,16 +184,17 @@ def contourplot(fvar, region=None, over=False, qual=''):
         raise ValueError('Ferret shade command (%s) failed: %s' % (cmdstr, errmsg))
 
 
-def fillplot(fvar, line=False, region=None, over=False, qual=''):
+def fillplot(fvar, region=None, line=False, over=False, pal=None, qual=''):
     """
     Create a color-filled contour plot of the specified Ferret variable using the Ferret 
     FILL command.  Drawing of the contour lines themselves is optional.
     The variable needs to be 2D (or qualifiers need to be added to specify a 2D slice).
         fvar (string or FerVar): Ferret variable to plot
-        line (bool): draw the contour lines?
         region (FerRegion): space-time region to plot; 
                 if None, the full extents of the data will be used
+        line (bool): draw the contour lines?
         over (bool): overlay on an existing plot?
+        pal (string): color palette to use
         qual (string): qualifiers to add to the Ferret SHADE command
     """
     if not isinstance(qual, str):
@@ -193,6 +214,8 @@ def fillplot(fvar, line=False, region=None, over=False, qual=''):
         if not isinstance(region, pyferret.FerRegion):
             raise ValueError('region, if given, must be a FerRegion')
         cmdstr += region._ferretqualifierstr();
+    if pal is not None:
+        cmdstr += '/PALETTE=' + str(pal)
     if qual:
         cmdstr += qual
     cmdstr += ' '
@@ -202,7 +225,7 @@ def fillplot(fvar, line=False, region=None, over=False, qual=''):
         raise ValueError('Ferret shade command (%s) failed: %s' % (cmdstr, errmsg))
 
 
-def shadeplot(fvar, region=None, over=False, qual=''):
+def shadeplot(fvar, region=None, over=False, pal=None, qual=''):
     """
     Create a colored plot of the specified Ferret variable using the Ferret SHADE command.
     (Plot coloring grid cells based on the variable value in that cell.)
@@ -211,6 +234,7 @@ def shadeplot(fvar, region=None, over=False, qual=''):
         region (FerRegion): space-time region to plot; 
                 if None, the full extents of the data will be used
         over (bool): overlay on an existing plot?
+        pal (string): color palette to use
         qual (string): qualifiers to add to the Ferret SHADE command
     """
     if not isinstance(qual, str):
@@ -228,6 +252,8 @@ def shadeplot(fvar, region=None, over=False, qual=''):
         if not isinstance(region, pyferret.FerRegion):
             raise ValueError('region, if given, must be a FerRegion')
         cmdstr += region._ferretqualifierstr();
+    if pal is not None:
+        cmdstr += '/PALETTE=' + str(pal)
     if qual:
         cmdstr += qual
     cmdstr += ' '
@@ -332,11 +358,12 @@ def shadewater(res=20, color='gray', over=True, solid=True, X=None, Y=None):
 
 
 def pointplot(fvar, vs=None, color=None, sym=None, symsize=None, thick=None,
-             line=False, title=None, region=None, over=False, label=True, qual=''):
+              line=False, title=None, region=None, over=False, label=True, qual=''):
     """
-    Create a point plot of the given value, or the given value versus another value (if vs is given),
-    possibly colored by another value (if color is a FerVar).
-    To create a line plot with symbols, use the pointplot command with the line option set to True.
+    Create a point plot of the given value, or the given value versus another value 
+    (if vs is given), possibly colored by another value (if color is a FerVar).
+    To create a line plot with symbols, use the pointplot command with the line 
+    option set to True.
         fvar (string or FerVar): Ferret variable to plot
         vs  (string or FerVar): if given, plot the above variable versus this variables
         color: line color or variable used to determine line color; if
@@ -433,11 +460,12 @@ def pointplot(fvar, vs=None, color=None, sym=None, symsize=None, thick=None,
 
 
 def lineplot(fvar, vs=None, color=None, thick=None, dash=None, title=None, 
-             region=None, over=False, label=True, qual=''):
+             region=None, along=None, over=False, label=True, qual=''):
     """
-    Create a line plot of the given value, or the given value versus another value (if vs is given),
-    possibly colored by another value (if color is a FerVar).
-    To create a line plot with symbols, use the pointplot command with the line option set to True.
+    Create a line plot of the given value, or the given value versus another value 
+    (if vs is given), possibly colored by another value (if color is a FerVar).
+    To create a line plot with symbols, use the pointplot command with the line 
+    option set to True.
         fvar (string or FerVar): Ferret variable to plot
         vs  (string or FerVar): if given, plot the above variable versus this variables
         color: line color or variable used to determine line color; if
@@ -455,6 +483,8 @@ def lineplot(fvar, vs=None, color=None, thick=None, dash=None, title=None,
         title (string): title for the plot; if not given,  Ferret's default title is used
         region (FerRegion): space-time region to plot; 
                 if None, the full extents of the data will be used
+        along (string; one of 'X','Y','Z','T','E','F', or lowercase): make a set of line 
+                plots from two-dimensional data with this axis as the horizontal axis.
         over (bool): overlay onto an existing plot
         label (bool): if False, suppress all plot labels
         qual (string): qualifiers to add to the Ferret PLOT/LINE command
@@ -503,11 +533,16 @@ def lineplot(fvar, vs=None, color=None, thick=None, dash=None, title=None,
        if not isinstance(title, str):
            raise ValueError('title must be a string')
        cmdstr += '/TITLE="' + title + '"'
+    if along is not None:
+       axisnames = ('X','Y','Z','T','E','F','x','y','z','t','e','f')
+       if not along in axisnames:
+           raise ValueError('along must be one of ' + str(axisnames))
+       cmdstr += '/ALONG=' + along.upper()
     if over:
         cmdstr += '/OVER'
     if region is not None:
         if not isinstance(region, pyferret.FerRegion):
-            raise ValueError('region, if given, must be a FerRegion')
+            raise ValueError('region must be a FerRegion')
         cmdstr += region._ferretqualifierstr();
     if not label:
         cmdstr += '/NOLABEL'
