@@ -167,7 +167,7 @@ class FerVar(object):
             they have the same Ferret variable name,
             they have the same dataset name, 
             they have the same title, and
-            they have the same defintion.
+            they have the same definition.
         All these comparisons are case-insensitive.
         '''
         try:
@@ -179,10 +179,10 @@ class FerVar(object):
     def __ne__(self, other):
         '''
         Two FerVars are not equal if any of the following are True:
-            they have different Ferret variable name,
-            they have different dataset name,
-            they have different title, or
-            they have different defintion.
+            they have different Ferret variable names,
+            they have different dataset names,
+            they have different titles, or
+            they have different definitions.
         All these comparisons are case-insensitive.
         '''
         try:
@@ -585,7 +585,7 @@ class FerVar(object):
         changed = False
         # TODO: handle step values, try to condense code
         if isinstance(key, tuple):
-            for k in xrange(len(key)):
+            for k in range(len(key)):
                 piece = key[k]
                 if piece is None:
                     continue
@@ -676,7 +676,7 @@ class FerVar(object):
                         val += 1
                         indexlimits[k] = '%d' % val
                         changed = True
-                    elif isinstance(start,float):
+                    elif isinstance(val,numbers.Real):
                         if coordlimits[k] or indexlimits[k]:
                             raise KeyError('two slices for axis index %d given' % k)
                         coordlimits[k] = '%s' % str(val)
@@ -746,7 +746,7 @@ class FerVar(object):
                 val += 1
                 indexlimits[k] = '%d' % val
                 changed = True
-            elif isinstance(start,float):
+            elif isinstance(start,numbers.Real):
                 coordlimits[k] = '%s' % str(val)
                 changed = True
             else:
@@ -855,7 +855,7 @@ class FerVar(object):
         # Assign the variable in Ferret
         cmdstr = 'DEFINE VAR'
         if dsetname:
-            cmdstr += '/D=%s' % dsetname
+            cmdstr += '/D="%s"' % dsetname
         if self._title:
             cmdstr += '/TITLE="%s"' % self._title
         cmdstr += ' %s = %s' % (varname, self._definition)
