@@ -384,14 +384,24 @@ class AbstractPyFerretBindings(object):
         '''
         raise AttributeError('not implemented')
 
-    def createSymbol(self, symbolname):
+    def createSymbol(self, symbolname, ptsx=None, ptsy=None):
         '''
         Returns a Symbol object.
 
         Arguments:
-            symbolname: name of the symbol (e.g., ".", "+")
+            symbolname: name of the symbol, either a well-known
+                  symbol name (e.g., '.') or a custom name for a 
+                  symbol created from the given vertices (e.g., 'FER001')
+            ptsx: vertices X-coordinates describing the symbol 
+                  as a multiline drawing on a [0,100] square; 
+                  not used if a well-known symbol name is given
+            ptsy: vertices Y-coordinates describing the symbol 
+                  as a multiline drawing on a [0,100] square; 
+                  not used if a well-known symbol name is given
 
-        Raises an error if unable to create the Symbol object.
+        Raises an error if ptsx and ptsy are needed but are not
+        sequence types of the same size or if unable to create 
+        the Symbol object for any other reason.
         '''
         raise AttributeError('not implemented')
 
