@@ -49,6 +49,7 @@
 #include "grdel.h"
 #include "pyferret.h"
 #include "pplmem.h"
+#include "FerMem.h"
 
 /* For older versions of NumPy (v1.4 with RHEL6), define this flag as the deprecated flag */
 #ifndef NPY_ARRAY_OWNDATA
@@ -73,16 +74,16 @@ PyObject *pyferret_graphbind_module_pyobject = NULL;
 /* Length given to the abstract axis */
 #define ABSTRACT_AXIS_LEN 9999999
 
+/* 
+ * String used as the missing value for String arrays in Python.
+ * Note that this is NULL ('\0') terminated. 
+ */
+static char *PYTHON_STRING_MISSING_VALUE = "\004";
+
 /* Flag of this Ferret's start/stop state */
 static int ferretInitialized = 0;
 
-/*
- * for memory management in this module
- * double *, instead of DFTYPE *, is used for ferMemory 
- * to create a warning if DFTYPE is not double.
- */
-static size_t ferMemSize;
-static double *ferMemory = NULL;
+/* Memory for PPLUS */
 static float  *pplMemory = NULL;
 
 /* for recovering from problems in external function calls */
@@ -249,6 +250,7 @@ static int assign_ferret_signal_handlers(void)
     return 0;
 }
 
+
 static char pyferretStartDocstring[] =
     "Initializes Ferret.  This allocates the initial amount of memory for \n"
     "Ferret (from Python-managed memory), opens the journal file, if requested, \n"
@@ -379,7 +381,7 @@ static PyObject *pyferretStart(PyObject *self, PyObject *args, PyObject *kwds)
 
     /* Initial allocation of PPLUS memory */
     pplMemSize = 0.5 * 1.0E6;
-    pplMemory = (float *) PyMem_Malloc((size_t)pplMemSize * (size_t)sizeof(float));
+    pplMemory = (float *) FerMem_Malloc((size_t)pplMemSize * (size_t)sizeof(float), __FILE__, __LINE__);
     if ( pplMemory == NULL )
         return PyErr_NoMemory();
     set_ppl_memory(pplMemory, pplMemSize);
@@ -406,7 +408,7 @@ static PyObject *pyferretStart(PyObject *self, PyObject *args, PyObject *kwds)
     FORTRAN(fgd_set_transparency)(&transparentFlag);
 
     /* Initialize stuff: keyboard, todays date, grids, GFDL terms, PPL brain */
-    FORTRAN(initialize)();
+    FORTRAN(initialize_ferret)();
 
     /* Open the output journal file, if appropriate */
     if ( journalFlag != 0 ) {
@@ -474,8 +476,8 @@ static PyObject *pyferretStart(PyObject *self, PyObject *args, PyObject *kwds)
 void reallo_ppl_memory(int new_size)
 {
     if ( pplMemory != NULL )
-        PyMem_Free(pplMemory);
-    pplMemory = (float *) PyMem_Malloc((size_t)new_size * sizeof(float));
+        FerMem_Free(pplMemory, __FILE__, __LINE__);
+    pplMemory = (float *) FerMem_Malloc((size_t)new_size * sizeof(float), __FILE__, __LINE__);
     if ( pplMemory == NULL ) {
         printf("Unable to allocate the requested %d words of PLOT memory.\n", new_size);
         exit(1);
@@ -618,14 +620,35 @@ static PyObject *pyferretRunCommand(PyObject *self, PyObject *args, PyObject *kw
     }
 
     if ( sBuffer->flags[FRTN_ACTION] == FACTN_EXIT ) {
-        /* plain "EXIT" Ferret command - instigate an orderly shutdown */
-
-        /* return Py_BuildValue("is", FERR_EXIT_PROGRAM, "EXIT"); */
-
-        /*
+        /* 
+         * plain "EXIT" Ferret command - exit completely
+         *
          * python -i -c ... intercepts the Python sys.exit() call and stays in python,
-         * so just do a C exit() from python
+         * so just do a C exit() from python - 
+         * after doing some clean-up for memory leak detection.
          */
+        if ( ferretInitialized ) {
+            /* Set to uninitialized */
+            ferretInitialized = 0;
+
+            /* Release the references to the pyferret and pyferret.graphbind modules */
+            Py_DECREF(pyferret_graphbind_module_pyobject);
+            pyferret_graphbind_module_pyobject = NULL;
+            Py_DECREF(pyferret_module_pyobject);
+            pyferret_module_pyobject = NULL;
+
+            /* Free memory allocated inside Ferret */
+            FORTRAN(finalize_ferret)();
+
+            /* Free memory allocated for PPLUS */
+            FerMem_Free(pplMemory, __FILE__, __LINE__);
+            pplMemory = NULL;
+        }
+
+#ifdef MEMORYDEBUG
+        (void) ReportAnyMemoryLeaks();
+#endif
+
         exit(0);
     }
 
@@ -1358,8 +1381,8 @@ static PyObject *pyferretGetStrData(PyObject *self, PyObject *args, PyObject *kw
     factor = 8 / sizeof(char *);
 
     /* Get the maximum string length of all the strings in this array */
-    /* Use STRING_MISSING_VALUE for missing strings */
-    maxstrlen = (int)(strlen(STRING_MISSING_VALUE) + 1);
+    /* Use PYTHON_STRING_MISSING_VALUE for missing strings */
+    maxstrlen = (int)(strlen(PYTHON_STRING_MISSING_VALUE) + 1);
     for (n = 0; n < (int)(shape[5]); n++) {
       for (m = 0; m < (int)(shape[4]); m++) {
         for (l = 0; l < (int)(shape[3]); l++) {
@@ -1416,7 +1439,7 @@ static PyObject *pyferretGetStrData(PyObject *self, PyObject *args, PyObject *kw
                                      m * strides[4] +
                                      n * strides[5] ) ];
                 if ( strptr == NULL ) {
-                   strptr = STRING_MISSING_VALUE;
+                   strptr = PYTHON_STRING_MISSING_VALUE;
                 }
                 /* because of +1 to strlen, these will always be null-terminated */
                 strncpy(&(npydata[q]), strptr, maxstrlen);
@@ -1439,7 +1462,7 @@ static PyObject *pyferretGetStrData(PyObject *self, PyObject *args, PyObject *kw
     }
     /* PyArray_Empty steals the reference to strarraydescript so do not free or reuse */
     npydata = (char *)PyArray_DATA(badval_ndarray);
-    strncpy(npydata, STRING_MISSING_VALUE, maxstrlen);
+    strncpy(npydata, PYTHON_STRING_MISSING_VALUE, maxstrlen);
 
     /* Create the axis coordinates array objects */
     for (k = 0; k < MAX_FERRET_NDIM; k++) {
@@ -1576,11 +1599,10 @@ static PyObject *pyferretGetStrData(PyObject *self, PyObject *args, PyObject *kw
 
 
 static char pyferretStopDocstring[] =
-    "Runs a series of Ferret commands to return Ferret to \n"
-    "its default state, then shuts down and releases all \n"
-    "memory used by Ferret.  After calling this function do \n"
-    "not call any Ferret functions except start, which will \n"
-    "restart Ferret and re-enable the other functions. \n"
+    "Returns Ferret to its default state, shuts down Ferret, and \n"
+    "releases all memory used by Ferret.  After calling this \n"
+    "function do not call any Ferret functions except start, \n"
+    "which will restart Ferret and re-enable the other functions. \n"
     "\n"
     "Required arguments: \n"
     "    (none) \n"
@@ -1589,7 +1611,7 @@ static char pyferretStopDocstring[] =
     "    (none) \n"
     "\n"
     "Returns: \n"
-    "    False if Ferret has not been started or has already been stopped \n"
+    "    False if Ferret has not been started or has already been stopped; \n"
     "    True otherwise \n";
 
 static PyObject *pyferretStop(PyObject *self)
@@ -1609,26 +1631,17 @@ static PyObject *pyferretStop(PyObject *self)
     Py_DECREF(pyferret_module_pyobject);
     pyferret_module_pyobject = NULL;
 
-    /* Run commands to clear/reset Ferret's state */
-    ferret_dispatch_c("SET GRID ABSTRACT", sBuffer);
-    ferret_dispatch_c("CANCEL WINDOW /ALL", sBuffer);
-    ferret_dispatch_c("CANCEL VARIABLE /ALL", sBuffer);
-    ferret_dispatch_c("CANCEL SYMBOL /ALL", sBuffer);
-    ferret_dispatch_c("CANCEL DATA /ALL", sBuffer);
-    ferret_dispatch_c("CANCEL REGION /ALL", sBuffer);
-    ferret_dispatch_c("CANCEL MEMORY /ALL", sBuffer);
-    ferret_dispatch_c("EXIT /PROGRAM", sBuffer);
+    /* Clear/reset Ferret's state and free memory allocated inside Ferret */
+    FORTRAN(finalize_ferret)();
 
-    /* Free memory allocated inside Ferret */
-    FORTRAN(finalize)();
-
-    /* Free memory allocated for Ferret */
-    PyMem_Free(ferMemory);
-    ferMemory = NULL;
-    ferMemSize = 0;
-    PyMem_Free(pplMemory);
+    /* Free memory allocated for PPLUS */
+    FerMem_Free(pplMemory, __FILE__, __LINE__);
     pplMemory = NULL;
 
+#ifdef MEMORYDEBUG
+    (void) ReportAnyMemoryLeaks();
+#endif
+
     /* Return True */
     Py_INCREF(Py_True);
     return Py_True;
@@ -1636,9 +1649,15 @@ static PyObject *pyferretStop(PyObject *self)
 
 
 static char pyferretQuitDocstring[] =
-    "Shuts down and release all memory used by Ferret. \n"
-    "This function is intended to be used with the atexit module \n"
-    "to ensure an open viewer window does not hang Python shutdown. \n"
+    "Returns Ferret to its default state, shuts down Ferret, and \n"
+    "releases all memory used by Ferret.  After calling this \n"
+    "function do not call any Ferret functions except start, \n"
+    "which will restart Ferret and re-enable the other functions. \n"
+    "\n"
+    "Except for the return value, this function is now identical \n"
+    "to the _stop method.  This method is retained for backwards \n"
+    "compatibility - to be used with the atexit module to ensure \n"
+    "an open viewer window does not hang Python shutdown. \n"
     "\n"
     "Required arguments: \n"
     "    (none) \n"
@@ -1651,35 +1670,13 @@ static char pyferretQuitDocstring[] =
 
 static PyObject *pyferretQuit(PyObject *self)
 {
-    /* If not initialized, nothing to do; just return None */
-    if ( ! ferretInitialized ) {
-        Py_INCREF(Py_None);
-        return Py_None;
-    }
+    PyObject *result;
 
-    /* Set to uninitialized */
-    ferretInitialized = 0;
+    /* Just call the _stop method */
+    result = pyferretStop(self);
+    Py_DECREF(result);
 
-    /* Release the references to the pyferret and pyferret.graphbind modules */
-    Py_DECREF(pyferret_graphbind_module_pyobject);
-    pyferret_graphbind_module_pyobject = NULL;
-    Py_DECREF(pyferret_module_pyobject);
-    pyferret_module_pyobject = NULL;
-
-    /* Let Ferret do its orderly shutdown - including closing viewers */
-    ferret_dispatch_c("EXIT /PROGRAM", sBuffer);
-
-    /* Free memory allocated inside Ferret */
-    FORTRAN(finalize)();
-
-    /* Free memory allocated for Ferret */
-    PyMem_Free(ferMemory);
-    ferMemory = NULL;
-    ferMemSize = 0;
-    PyMem_Free(pplMemory);
-    pplMemory = NULL;
-
-    /* Return None */
+    /* But return None */
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -2317,7 +2314,7 @@ static void AddConstantsToPyFerret(PyObject *mod)
     PyModule_AddIntConstant(mod, "MAX_FERRET_NDIM", MAX_FERRET_NDIM);
 
     /* Parameter giving the String used as the missing value for String arrays */
-    PyModule_AddStringConstant(mod, "STRING_MISSING_VALUE", STRING_MISSING_VALUE);
+    PyModule_AddStringConstant(mod, "STRING_MISSING_VALUE", PYTHON_STRING_MISSING_VALUE);
 
     /* Private parameter return value from libpyferret._run indicating the program should shut down */
     PyModule_AddIntConstant(mod, "_FERR_EXIT_PROGRAM", FERR_EXIT_PROGRAM);
