@@ -34,8 +34,10 @@
 
 #include <Python.h>
 #define PY_ARRAY_UNIQUE_SYMBOL pyferret_ARRAY_API
+#define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION
 #define NO_IMPORT_ARRAY
 #include <numpy/arrayobject.h>
+#include "ferret.h"
 #include "pyferret.h"
 #include "EF_Util.h"
 
@@ -74,7 +76,11 @@ void pyefcn_init(int id, char modname[], char errmsg[])
     /*
      * Import the user's Python module
      */
+#if PY_MAJOR_VERSION > 2
+    valobj = PyUnicode_FromString(modname);
+#else
     valobj = PyString_FromString(modname);
+#endif
     if ( valobj == NULL ) {
         PyErr_Clear();
         sprintf(errmsg, "Problems creating a Python string from the module name: %s", modname);
@@ -123,14 +129,18 @@ void pyefcn_init(int id, char modname[], char errmsg[])
                         INIT_METHOD_NAME, modname);
         return;
     }
+#if PY_MAJOR_VERSION > 2
+    num_args = (int) PyLong_AsLong(valobj);
+#else
     num_args = (int) PyInt_AsLong(valobj);
+#endif
     if ( (num_args < 1) || (num_args > EF_MAX_ARGS) ) {
         PyErr_Clear();
         Py_DECREF(initdict);
         strcpy(errmsg, "Invalid \"numargs\" value (not an integer [0-9])");
         return;
     }
-    ef_set_num_args_(&id, &num_args);
+    FORTRAN(ef_set_num_args)(&id, &num_args);
 
     /*
      * "descript": string description of the function [required]
@@ -142,7 +152,11 @@ void pyefcn_init(int id, char modname[], char errmsg[])
                         INIT_METHOD_NAME, modname);
         return;
     }
+#if PY_MAJOR_VERSION > 2
+    strptr = PyUnicode_AsUTF8(valobj);
+#else
     strptr = PyString_AsString(valobj);
+#endif
     if ( strptr == NULL ) {
         PyErr_Clear();
         Py_DECREF(initdict);
@@ -151,14 +165,18 @@ void pyefcn_init(int id, char modname[], char errmsg[])
     }
     strncpy(descript, strptr, EF_MAX_DESCRIPTION_LENGTH);
     descript[EF_MAX_DESCRIPTION_LENGTH - 1] = '\0';
-    ef_set_desc_sub_(&id, descript);
+    FORTRAN(ef_set_desc_sub)(&id, descript);
 
     /*
      * "restype": type of the result argument [optional, default: FLOAT_ARRAY]
      */
     valobj = PyDict_GetItemString(initdict, "restype"); /* borrowed reference */
     if ( valobj != NULL ) {
+#if PY_MAJOR_VERSION > 2
+        restype = (int) PyLong_AsLong(valobj);
+#else
         restype = (int) PyInt_AsLong(valobj);
+#endif
         if ( restype == FLOAT_ARRAY ) {
             restype = FLOAT_RETURN;
         }
@@ -176,7 +194,7 @@ void pyefcn_init(int id, char modname[], char errmsg[])
         /* Key not present but no error raised */
         restype = FLOAT_RETURN;
     }
-    ef_set_result_type_(&id, &restype);
+    FORTRAN(ef_set_result_type)(&id, &restype);
 
     /*
      * "resstrlen": (maximum) length of strings in the result string array [optional]
@@ -184,7 +202,11 @@ void pyefcn_init(int id, char modname[], char errmsg[])
      */
     valobj = PyDict_GetItemString(initdict, "resstrlen"); /* borrowed reference */
     if ( valobj != NULL ) {
+#if PY_MAJOR_VERSION > 2
+        resstrlen = (int) PyLong_AsLong(valobj);
+#else
         resstrlen = (int) PyInt_AsLong(valobj);
+#endif
         if ( resstrlen < 1 ) {
             PyErr_Clear();
             Py_DECREF(initdict);
@@ -230,7 +252,12 @@ void pyefcn_init(int id, char modname[], char errmsg[])
         rsltaxes[k] = IMPLIED_BY_ARGS;
         if ( k < seqlen ) {
             itemobj = PySequence_Fast_GET_ITEM(seqobj, (Py_ssize_t) k); /* borrowed reference */
-            switch( (int) PyInt_AsLong(itemobj) ) {
+#if PY_MAJOR_VERSION > 2
+            val = (int) PyLong_AsLong(itemobj);
+#else
+            val = (int) PyInt_AsLong(itemobj);
+#endif
+            switch( val ) {
                 case IMPLIED_BY_ARGS:
                     break;
                 case ABSTRACT:
@@ -262,9 +289,9 @@ void pyefcn_init(int id, char modname[], char errmsg[])
         }
     }
     Py_XDECREF(seqobj);
-    ef_set_axis_inheritance_6d_(&id, &(axisvals[0]), &(axisvals[1]), &(axisvals[2]), 
+    FORTRAN(ef_set_axis_inheritance_6d)(&id, &(axisvals[0]), &(axisvals[1]), &(axisvals[2]), 
                                      &(axisvals[3]), &(axisvals[4]), &(axisvals[5]));
-    ef_set_axis_reduction_6d_(&id, &(axisredu[0]), &(axisredu[1]), &(axisredu[2]), 
+    FORTRAN(ef_set_axis_reduction_6d)(&id, &(axisredu[0]), &(axisredu[1]), &(axisredu[2]), 
                                    &(axisredu[3]), &(axisredu[4]), &(axisredu[5]));
 
     /*
@@ -311,7 +338,7 @@ void pyefcn_init(int id, char modname[], char errmsg[])
         }
     }
     Py_XDECREF(seqobj);
-    ef_set_piecemeal_ok_6d_(&id, &(axisvals[0]), &(axisvals[1]), &(axisvals[2]), 
+    FORTRAN(ef_set_piecemeal_ok_6d)(&id, &(axisvals[0]), &(axisvals[1]), &(axisvals[2]), 
                                  &(axisvals[3]), &(axisvals[4]), &(axisvals[5]));
 
     /*
@@ -345,7 +372,11 @@ void pyefcn_init(int id, char modname[], char errmsg[])
         if ( j < seqlen ) {
             /* Get the name from the tuple */
             itemobj = PySequence_Fast_GET_ITEM(seqobj, (Py_ssize_t) j); /* borrowed reference */
+#if PY_MAJOR_VERSION > 2
+            strptr = PyUnicode_AsUTF8(itemobj);
+#else
             strptr = PyString_AsString(itemobj);
+#endif
             if ( strptr != NULL ) {
                 strncpy(name, strptr, EF_MAX_NAME_LENGTH);
                 name[EF_MAX_NAME_LENGTH-1] = '\0';
@@ -358,7 +389,7 @@ void pyefcn_init(int id, char modname[], char errmsg[])
             }
         }
         q = j+1;
-        ef_set_arg_name_sub_(&id, &q, name);
+        FORTRAN(ef_set_arg_name_sub)(&id, &q, name);
     }
     Py_XDECREF(seqobj);
 
@@ -392,7 +423,11 @@ void pyefcn_init(int id, char modname[], char errmsg[])
         if ( j < seqlen ) {
             /* Get the description from the tuple */
             itemobj = PySequence_Fast_GET_ITEM(seqobj, (Py_ssize_t) j); /* borrowed reference */
+#if PY_MAJOR_VERSION > 2
+            strptr = PyUnicode_AsUTF8(itemobj);
+#else
             strptr = PyString_AsString(itemobj);
+#endif
             if ( strptr != NULL ) {
                 strncpy(descript, strptr, EF_MAX_DESCRIPTION_LENGTH);
                 descript[EF_MAX_DESCRIPTION_LENGTH-1] = '\0';
@@ -406,7 +441,7 @@ void pyefcn_init(int id, char modname[], char errmsg[])
             }
         }
         q = j + 1;
-        ef_set_arg_desc_sub_(&id, &q, descript);
+        FORTRAN(ef_set_arg_desc_sub)(&id, &q, descript);
     }
     Py_XDECREF(seqobj);
 
@@ -446,7 +481,12 @@ void pyefcn_init(int id, char modname[], char errmsg[])
         if ( j < seqlen ) {
             /* Get the type from the tuple */
             itemobj = PySequence_Fast_GET_ITEM(seqobj, (Py_ssize_t) j); /* borrowed reference */
-            switch( (int) PyInt_AsLong(itemobj) ) {
+#if PY_MAJOR_VERSION > 2
+            val = (int) PyLong_AsLong(itemobj);
+#else
+            val = (int) PyInt_AsLong(itemobj);
+#endif
+            switch( val ) {
                 case FLOAT_ONEVAL:
                 case FLOAT_ARRAY:
                 case FLOAT_ARG:
@@ -465,7 +505,7 @@ void pyefcn_init(int id, char modname[], char errmsg[])
             }
         }
         q = j+1;
-        ef_set_arg_type_(&id, &q, &argtype);
+        FORTRAN(ef_set_arg_type)(&id, &q, &argtype);
     }
     Py_XDECREF(seqobj);
 
@@ -549,7 +589,7 @@ void pyefcn_init(int id, char modname[], char errmsg[])
             }
         }
         q = j+1;
-        ef_set_axis_influence_6d_(&id, &q, &(axisvals[0]), &(axisvals[1]), &(axisvals[2]), 
+        FORTRAN(ef_set_axis_influence_6d)(&id, &q, &(axisvals[0]), &(axisvals[1]), &(axisvals[2]), 
                                            &(axisvals[3]), &(axisvals[4]), &(axisvals[5]));
     }
     Py_XDECREF(seqobj);
@@ -625,7 +665,11 @@ void pyefcn_init(int id, char modname[], char errmsg[])
                         deltas[0] = 0; deltas[1] = 0;
                         for (q = 0; q < subsubseqlen; q++) {
                             itemobj = PySequence_Fast_GET_ITEM(subsubseqobj, (Py_ssize_t) q); /* borrowed reference */
+#if PY_MAJOR_VERSION > 2
+                            val = PyLong_AsLong(itemobj);
+#else
                             val = PyInt_AsLong(itemobj);
+#endif
                             if ( PyErr_Occurred() ) {
                                 PyErr_Clear();
                                 Py_DECREF(subsubseqobj);
@@ -653,7 +697,7 @@ void pyefcn_init(int id, char modname[], char errmsg[])
                         }
                         q = j+1;
                         val = k+1;
-                        ef_set_axis_extend_(&id, &q, &val, &(deltas[0]), &(deltas[1]));
+                        FORTRAN(ef_set_axis_extend)(&id, &q, &val, &(deltas[0]), &(deltas[1]));
                         Py_DECREF(subsubseqobj);
                     }
                 }
@@ -669,7 +713,11 @@ void pyefcn_init(int id, char modname[], char errmsg[])
     seqlen = (int) PySequence_Fast_GET_SIZE(seqobj);
     for (k = 0; k < seqlen; k++) {
         itemobj = PySequence_Fast_GET_ITEM(seqobj, (Py_ssize_t) k); /* borrowed reference */
+#if PY_MAJOR_VERSION > 2
+        strptr = PyUnicode_AsUTF8(itemobj);
+#else
         strptr = PyString_AsString(itemobj);
+#endif
         if ( strptr == NULL ) {
             Py_DECREF(seqobj);
             Py_DECREF(keysobj);
